#include"context.hpp"
#include"mouse.hpp"




namespace oat{
namespace context{


namespace{


Mouse
mouse;


Root*  current_root;


std::list<std::unique_ptr<Root> >  root_list;


}


SDL_Surface*  screen;

uint8_t*  pixels;

int  pitch;


void
insert(Root*  root)
{
  root_list.emplace_front(std::unique_ptr<Root>(root));

    if(!current_root)
    {
      current_root = root;

      root->enter();
    }
}


void
reenter(const SDL_Window*  win)
{
    if(*current_root != win)
    {
      current_root->leave();

        for(auto&  ptr: root_list)
        {
            if(*ptr == win)
            {
              current_root = ptr.get();

              current_root->enter();

              break;
            }
        }
    }
}


Root&
get_current_root()
{
  return*current_root;
}




Root*
find(uint32_t  id)
{
    for(auto&  ptr: root_list)
    {
        if(ptr->test_window_id(id))
        {
          return ptr.get();
        }
    }


  return nullptr;
}


void
process_mouse(Root*  root)
{
    if(current_root == root)
    {
      auto&  current = root->get_current_widget();

        if(mouse.test_enter(current))
        {
          mouse.reset_rel(current);

          current.process(mouse);
        }

      else
        {
          root->process(mouse);
        }
    }

  else
    {
      reenter(root->get_window());

      root->process(mouse);
    }
}


void
button(const SDL_MouseButtonEvent&  evt)
{
  auto  root = find(evt.windowID);

    if(root)
    {
      mouse.button(evt);

      process_mouse(root);
    }
}


void
motion(const SDL_MouseMotionEvent&  evt)
{
  auto  root = find(evt.windowID);

    if(root)
    {
      mouse.motion(evt);

      process_mouse(root);
    }
}


void
process(const SDL_KeyboardEvent&  evt)
{
  auto  root = find(evt.windowID);

    if(root)
    {
      root->get_current_widget().process(evt);
    }
}


void
drop(char*  s)
{
    if(current_root)
    {
      current_root->do_on_drop(s);
    }


  SDL_free(s);
}


void
update()
{
  auto   it = root_list.begin();
  auto  end = root_list.end();

    while(it != end)
    {
        if((*it)->test_flag(WID_ST_destruct_me_flag))
        {
          it = root_list.erase(it);
        }

      else
        {
          (*it)->update();

          ++it;
        }
    }
}


bool
process_event()
{
  static SDL_Event  evt;

    while(SDL_PollEvent(&evt))
    {
      Root*  root;

        switch(evt.type)
        {
          case(SDL_QUIT):
            return false;
            break;
          case(SDL_DROPFILE       ): drop(evt.drop.file);          break;
          case(SDL_MOUSEBUTTONUP  ):
          case(SDL_MOUSEBUTTONDOWN): button(evt.button);           break;
          case(SDL_MOUSEMOTION    ): motion(evt.motion);           break;
          case(SDL_KEYUP          ):
              if(evt.key.keysym.sym == SDLK_F5)
              {
                int  i = 0;

                  for(auto&  root: root_list)
                  {
                    char  buf[256];

                    snprintf(buf,sizeof(buf),"_OAT_WINDOW%02d.png",i++);

                    root->save_png(buf);
                  }
              }


          case(SDL_KEYDOWN):
            process(evt.key);
            break;
          case(SDL_WINDOWEVENT):
              switch(evt.window.event)
              {
                case(SDL_WINDOWEVENT_EXPOSED):
                case(SDL_WINDOWEVENT_MOVED):
                  root = find(evt.window.windowID);

                    if(root)
                    {
                      root->update();
                    }
                  break;
              }
            break;
          default:;
        }
    }


  current_root->get_current_widget().do_on_everytime();

  return true;
}


}}




