#include"root.hpp"
#include"context.hpp"
#include"surface.hpp"
#include"font.hpp"
#include"pngbuffer.hpp"




namespace oat{




Root::
Root(const std::string&  title, int  x, int  y):
current_widget(this),
cb(nullptr)
{
  widget_parameter.rel_point.x = 8;
  widget_parameter.rel_point.y = 8;

  root = this;

  window = SDL_CreateWindow(title.data(),x,y,320,32,0);

  renderer = SDL_CreateRenderer(window,-1,0);

  texture = nullptr;

  SDL_SetWindowData(window,"oat::widget",this);

  font::reset();

  reset_default_colors();

  back_color = gray_color;
}


Root::
~Root()
{
  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);
}




SDL_Window*
Root::
get_window()
{
  return window;
}


const SDL_Window*
Root::
get_window() const
{
  return window;
}


Widget&  Root::get_current_widget(){return *current_widget;}


bool
Root::
test_window_id(uint32_t  id)
{
  return(SDL_GetWindowID(window) == id);
}


bool
Root::
operator==(const SDL_Window*  win) const
{
  return(window == win);
}


bool
Root::
operator!=(const SDL_Window*  win) const
{
  return(window != win);
}




void
Root::
set_callback(Callback  _cb)
{
  cb = _cb;
}


bool
Root::
test_callback() const
{
  return cb;
}


void
Root::
do_on_drop(const char*  s)
{
    if(cb)
    {
      cb(*this,s);
    }

  else
    if(current_widget != this)
    {
      current_widget->do_on_drop(s);
    }
}


void
Root::
reenter(Widget&  wid)
{
    if(current_widget != &wid)
    {
      current_widget->leave();

      current_widget = &wid;

      current_widget->enter();
    }
}


void
Root::
save_png(const char*  path)
{
  PNGBuffer  buf;

  int  w;
  int  h;

  SDL_GetWindowSize(window,&w,&h);

  buf.make_direct_color(w,h);

  std::vector<uint8_t>  pbuf(4*w*h);

  context::pixels = pbuf.data();
  context::pitch  =       (4*w);

    rep(y,h)
    {
      auto  p = (uint32_t*)&context::pixels[context::pitch*y];

        rep(x,w)
        {
          *p++ = back_color;
        }
    }


  Node::draw();

    rep(y,h)
    {
      auto  p = (uint32_t*)&context::pixels[context::pitch*y];

        rep(x,w)
        {
          uint8_t  r;
          uint8_t  g;
          uint8_t  b;

          SDL_GetRGB(*p++,common_format,&r,&g,&b);

          auto  dst = buf.get_color(x,y);

          *dst++ = r;
          *dst++ = g;
          *dst++ = b;
        }
    }


  buf.save(path);
}


void
Root::
draw()
{
  void*  ptr;

  SDL_LockTexture(texture,nullptr,&ptr,&context::pitch);

  context::pixels = (uint8_t*)ptr;

    rep(y,widget_parameter.height+16)
    {
      auto  p = (uint32_t*)&context::pixels[context::pitch*y];

        rep(x,widget_parameter.width+16)
        {
          *p++ = back_color;
        }
    }


  Node::draw();

  SDL_UnlockTexture(texture);


  SDL_RenderClear(renderer);

  SDL_RenderCopy(renderer,texture,nullptr,nullptr);

  SDL_RenderPresent(renderer);
}


void
Root::
redraw()
{
  void*  ptr;

  SDL_LockTexture(texture,nullptr,&ptr,&context::pitch);

  context::pixels = (uint8_t*)ptr;

  Node::redraw();


  SDL_UnlockTexture(texture);


  SDL_RenderClear(renderer);

  SDL_RenderCopy(renderer,texture,nullptr,nullptr);

  SDL_RenderPresent(renderer);
}


void
Root::
update()
{
  destruct();

    if(test_flag(WID_ST_dirty_flag))
    {
      cleanup(0,0);

      const int  w = widget_parameter.right +8;
      const int  h = widget_parameter.bottom+8;

      texture = SDL_CreateTexture(renderer,context::format_code,SDL_TEXTUREACCESS_STREAMING,w,h);

      SDL_SetWindowSize(window,w,h);

      draw();
    }

  else
    {
      redraw();
    }
}




}




