#include"widget.hpp"
#include"root.hpp"
#include"context.hpp"
#include<cstring>




namespace oat{




namespace{


WidgetID
id_source;


std::vector<WidgetID>
id_trash;


uint32_t
get_new_id()
{
  uint32_t  id;

    if(id_trash.size())
    {
      id = id_trash.back();

      id_trash.pop_back();
    }

  else
    {
      id = id_source++;
    }


  return id;
}


void*  default_userdata;


}




void   Widget::set_default_userdata(void*  ptr){       default_userdata = ptr;}
void*  Widget::get_default_userdata(          ){return default_userdata      ;}




Widget::
Widget(WidgetType  type):
widget_type(type),
widget_id(get_new_id()),
parent(nullptr),
widget_flags(WID_ST_active_flag|WID_ST_visible_flag|WID_ST_dirty_flag),
userdata(default_userdata)
{
  std::memset(&widget_parameter,0,sizeof(widget_parameter));

  fore_color = 0xFFFFFFFF;
  back_color = gray_color;
}


Widget::
~Widget()
{
  id_trash.emplace_back(widget_id);
}




bool
Widget::
operator==(WidgetType  type) const
{
  return(widget_type == type);
}


bool
Widget::
operator!=(WidgetType  type) const
{
  return(widget_type != type);
}


void  Widget::set_name(const std::string&  name){widget_name = name;}


Node*  Widget::get_parent(){return parent;}


void   Widget::set_userdata(void*  data, bool  spread){       userdata = data;}
void*  Widget::get_userdata(                         ){return userdata       ;}


void
Widget::
cleanup(int  x, int  y)
{
  widget_parameter.top  = (y+widget_parameter.rel_point.y);
  widget_parameter.left = (x+widget_parameter.rel_point.x);

  widget_parameter.right  = (widget_parameter.left+widget_parameter.width );
  widget_parameter.bottom = (widget_parameter.top +widget_parameter.height);

  reset_flag(WID_ST_dirty_flag);
}


void
Widget::
join_to(Node&  _parent, int  x, int  y)
{
    if(!parent)
    {
      parent = &_parent;

      widget_parameter.rel_point.x = x;
      widget_parameter.rel_point.y = y;

      set_flag(WID_ST_dirty_flag);
    }
}


bool  Widget::test_flag(uint32_t  flag) const{return(widget_flags&flag);}


void
Widget::
set_flag(uint32_t  flag, bool  notify)
{
  widget_flags |= flag;

    if(notify && parent)
    {
      parent->set_flag(flag,true);
    }
}


void  Widget::reset_flag(uint32_t  flag){widget_flags &= ~flag;}


const WidgetParameter&
Widget::
get_parameter() const
{
  return widget_parameter;
}


const WidgetParameter*
Widget::
operator->() const
{
  return &widget_parameter;
}


void  Widget::set_fore_color(Color  color){fore_color = color;}
void  Widget::set_back_color(Color  color){back_color = color;}


void
Widget::
redraw()
{
    if(test_flag(WID_ST_redraw_flag))
    {
        if(test_flag(WID_ST_visible_flag))
        {
          draw();
        }


      reset_flag(WID_ST_redraw_flag);
    }
}


void
Widget::
request_redraw()
{
  set_flag(WID_ST_redraw_flag,true);
}


void
Widget::
enter()
{
  set_flag(WID_ST_current_flag);

  SDL_ShowCursor(test_flag(WID_ST_hide_cursor_flag)? 0:1);

  do_on_enter();
}


void
Widget::
leave()
{
  reset_flag(WID_ST_current_flag);

  do_on_leave();
}


void
Widget::
enable()
{
  set_flag(WID_ST_active_flag);
  set_flag(WID_ST_visible_flag);

  request_redraw();
}


void
Widget::
disable()
{
  reset_flag(WID_ST_active_flag);
  reset_flag(WID_ST_visible_flag);

  request_redraw();


  set_flag(WID_ST_fill_flag);
}




void
Widget::
print() const
{
  static int  n;

  printf("WIDGET PRINT %8d\n",n++);

    switch(widget_type)
    {
      case(WID_node  ): printf("[Node]\n");break;
      case(WID_area  ): printf("[Area]\n");break;
      case(WID_button): printf("[Button]\n");break;
      case(WID_label ): printf("[Label]\n");break;
      default:;
    }


  printf("%s\n",widget_name.data());

  printf("flags  ");

    if(test_flag(WID_ST_active_flag)){printf("active ");}
    if(test_flag(WID_ST_redraw_flag)){printf("redraw ");}
    if(test_flag(WID_ST_visible_flag)){printf("visible ");}


  printf("\n");

  widget_parameter.print();
}




}




